# Skypager::Builder::Server
#
# A Rack mountable server which can receive Webhook notifications
# from services like Dropbox, Google Drive, or our own Rest APIs
# and trigger builds for the various Skypager::Site that are referenced
# in the Skypager::Site.directory
#
# Currently, the Builder::Server just finds sites and marks them as requiring
# a build. Some external process will poll the sites directory to find the sites
# which require a build, and run that for us. (Cron, perhaps)
module Skypager
  class Builder
    class Server
      attr_reader :options

      def initialize(*args)
        @options = args.extract_options!
      end

      def call(env)
        request = Rack::Request.new(env)
        respond_to(request)
      rescue => e
        [500, {}, [e.message]]
      end

      private

      def webhooks
        require "skypager/builder/webhook_handler" unless defined?(Skypager::Builder::WebhookHandler)
        Skypager::Builder::WebhookHandler
      end

      def respond_to_webhook(request)
        case
        when request.path.match(/dropbox/)
          puts "== Found dropbox webhook"
          webhooks.handle(:dropbox, request)
        when request.path.match(/github/)
          puts "== Found github webhook"
          webhooks.handle(:github, request)
        when request.path.match(/google/)
          puts "== Found google webhook"
          webhooks.handle(:google, request)
        else
          puts "== Found custom webhook"
          webhooks.handle(:custom, request)
        end
      end

      def respond_to_directory_request(request)
        [200, {}, [{type:"directory"}.to_json]]
      end

      def respond_to(request)
        puts "== Responding to: #{ request.path }"
        case
        when request.path.match(/hooks/)
          puts "== Found hooks path"
          respond_to_webhook(request)
        when request.path.match(/directory/)
          puts "== Found directory path"
          respond_to_directory_request(request)
        else
          [404, {}, [""]]
        end
      end
    end
  end
end
