module Skypager::Builder::WebhookHandler
  def self.handle(service_identifier=:custom, request)
    unless respond_to?("handle_#{service_identifier}")
      return [404, {}, [""]]
    end

    puts "== Handling #{ service_identifier }: #{ request.path } #{ request.params }"
    send("handle_#{service_identifier}", request)
  end

  # The Dropbox API requires us to support
  # two different types of Webhook requests.
  #
  # One will include the UID of the users who have changes.
  # We will need to find all of the skypager sites which are
  # registered against this UID in order to find the credentials
  # needed to make a delta request to the API.
  #
  # We will then use the paths we find in the delta, to find
  # the specific site that we need to mark for building
  def self.handle_dropbox(request)
    puts "== Dropbox handler initiated"
    body = request.body.read

    puts "== Request body: #{ body }"
    puts "== Params: #{ request.params.inspect }"

    begin
      if request.params['challenge']
        puts "== challenge request"
        response = "#{request.params['challenge']}"
      elsif body.length > 0
        data = JSON.parse(body) rescue nil
        data = data.to_mash if data.respond_to?(:to_mash)

        Array(data.delta.users).each do |uid|
          puts "== Attempting to find a site for #{ uid }"
          sites = Skypager::Site.find_sites_for_dropbox_uid(uid)

          if sites.length > 0
            sites.each do |site|
              puts "== Marking site for build: #{ site.name }"
              site.requires_build!
            end
          else
            puts "== Could not find site for uid: #{ uid }"
          end
        end

        response = data.delta.users.to_json
      end
    rescue => e
      response = "== Error: #{ e.message }"
      puts response
    end

    [200, {}, [response]]
  end

  # TODO
  # Investigate the makeup of google drive webhook request
  def self.handle_google(request)
  end

  # TODO
  # Presumably we will get commit notifications from Github,
  # and will find the site that matches the notification
  def self.handle_github(request)
  end

  def self.handle_custom(request)
  end
end
