# The Skypager::Router provides a simple way for
# proxying requests to different skypager builds, or
# development environments.
module Skypager
  class Router
    attr_accessor :app

    def initialize(app, options={})
      @app = app
      @options = options
    end

    class << self
      attr_accessor :proxies
    end

    def to_s
      "Skypager Router. Serving #{ proxies.keys.join(", ") }"
    end

    def inspect
      to_s
    end

    def self.add(subdomain, options={})
      subdomain = subdomain.to_s
      self.proxies ||= {}
      self.proxies[subdomain] = Skypager.proxy(options)
    end

    def proxies
      self.class.proxies
    end

    def self.authenticator &block
      @authenticator = block if block_given?
      @authenticator
    end

    def authenticator
      self.class.authenticator
    end

    def call(env)
      request = Rack::Request.new(env)
      subdomain = request.host.split('.').first

      handler = proxies.fetch(subdomain) do |key|
        app
      end

      if authenticator.respond_to?(:call)
        if !!authenticator.call(env, subdomain)
          return [401, {}, [""]]
        end
      end

      handler.call(env)
    end
  end
end
